/*
 ********************************************************
	Rug Warrior Pro UMH-ISA (SITR)
 ********************************************************
	NOMBRE DEL ARCHIVO:   	comport.c
	FECHA:			
	AUTOR:   Luis M. Jimnez
	DESCRIPCIN:  Implementacin de diferentes comportamientos
					necesita: motores.c, comun.c, defines.c
					musica.c
				  
	VARIABLES GLOBALES EXTERNAS:
		int cruise_command = FORWARD;
		int photo_command = STOP;
		int ir_command = STOP;
		int bump_command = STOP;
		int sound_command = STOP;

		int cruise_active = TRUE;
		int photo_active = FALSE;
		int ir_active = FALSE;
		int bump_active = FALSE;
		int sound_active = FALSE;

		int sound_inhibit = 0;	
		int sleep_time = 100;

		int photo_follow=1;
		int photo_cal = -8;
		int photo_dead_zone = 2;
		int photo_out;
		int ir_out=0;

		int bump_out=0;

		int sound_thresh = 220;
		int sound_out;


    FUNCIONES EXTERNAS: 
		void motor_control();
		int cruise();
		int photo();
		void ir_avoid();
		void ir_follow()
		void bump();
		void printer();
		void sound ();
		void main();

	VARIABLES GLOBALES USO INTERNO:

    FUNCIONES USO INTERNO: 
 ********************************************************
*/

int cruise_command = FORWARD;
int photo_command = STOP;
int ir_command = STOP;
int bump_command = STOP;
int sound_command = STOP;

int cruise_active = TRUE;
int photo_active = FALSE;
int ir_active = FALSE;
int bump_active = FALSE;
int sound_active = FALSE;

int sound_inhibit = 0;		/* Use this flag to turn off some sounds */
int sleep_time = 100;		/* mseconds to sleep behavior */

/* The arbitration scheme is as follows:  Each behavior may compute an output
   for the motors.  It if does it stores the value in a global and sets an
   active flag.  The master motor controller resets the flags and sends the
   highest priorty command to the motor.
*/

void motor_control()
{  
	while (1) 
	{
      if (sound_active)
         move(sound_command);
      else if (bump_active)
         move(bump_command);
      else if (ir_active)
         move(ir_command);
      else if (photo_active)
         move(photo_command);
      else if (cruise_active)
	     move(cruise_command);
      else
	      move(STOP);	/* No commands => STOP */
      defer();			/* Update once per scheduler iteration */
	}
}

/* Default activity */
int cruise()
{ 
	while(1) {
		cruise_command = FORWARD;			/* Robot forward */
		cruise_active = TRUE;
		wait(1000);					/* Cruise once a second */
		/*msleep((long)10 * sleep_time);	/* Only every once in a while */
	}
}


/*  Seek (or avoid) a light source
 * Call with photo_follow = 1 to have robot seek light
 * photo_follow = -1 makes robot seek dark
 */
/* Bright light => low numbers  */
int photo_follow=1;		/* 1 follow light, -1 avoid light */
int photo_cal = -8;		/* calibration */
int photo_dead_zone = 2;	/* If the photo cells aren't equal... */
int photo_out;			/* State of photo */

int photo()
{   
	int lpc, rpc, delta;			/* Left and Right Photo Cells */
    while (1) {     
       lpc = analog(1) + photo_cal;
       rpc = analog(0);
       delta = (rpc - lpc)*photo_follow;	/* + => left sees bright, - => right */
	   
       if ( abs(delta) > photo_dead_zone )
		{
			if (delta > 0)
				photo_command = ARC_LEFT;	/* Left bright => turn left */
			else
				photo_command = ARC_RIGHT;	/* Otherwise turn right */
			photo_active = TRUE;			/* Activate when detected */
		}
	   else
			photo_active = FALSE;			/* Deactivate when not detected */

	   photo_out=delta;
	   defer();			/* Once per scheduler tick */	
		/* msleep(sleep_time);	/* limit activity */
    }
}


/* Check ir detector */
int ir_out=0;

/* AVOIDS OBJECTS */
void ir_avoid()
{ 
  int val;
  while (1) {
     val = ir_detect();
     if (val == IR_BOTH)	/* Left and right */
	 { 
		ir_active = TRUE;
		ir_command = ARC_LEFT;
		if (sound_inhibit == 0)
			tone(1500.,.05);  
	 }  
     else if (val == IR_LEFT)	/* Left IR */
	 { 
		ir_active = TRUE;
		ir_command = ARC_RIGHT;
		if (sound_inhibit == 0)
			tone(2000.,.05); 
	 }
     else if (val == IR_RIGHT)	/* Right IR */
	 { 
		ir_active = TRUE;
		ir_command = ARC_LEFT;
		if (sound_inhibit == 0)
			tone(1000.,.05);
	 }
     else
	 { 
		 ir_active = FALSE;
	 }

	 ir_out=val;
	/* msleep(sleep_time); } */
     defer();
    }
}


/* Use the IRs to follow closest target */

void ir_follow()
{ 
  int ir = 0;                   /* Local var for obstacle infrared data */
  while(1)
  { 
	  ir = ir_detect();         /* Record IR obstacle detection sensor */
      if (ir == IR_NONE)         /* Don't know which way to go, so stop */
		ir_active = FALSE;		 /* Stay right here */

      else if (ir == IR_BOTH)      /* Object straight a head */
	  {		
		ir_active = TRUE;
		ir_command = FORWARD;	 /* Race forward */
		if (sound_inhibit == 0)
			tone(1500.,.05);  
	  }
      else if (ir == IR_RIGHT)      /* Object on right */
	  {
		ir_active = TRUE;
		ir_command = ARC_RIGHT;	 /* Arc to the right */
		if (sound_inhibit == 0)
			tone(1000.,.05);
	  }
 
      else if (ir == IR_LEFT)      /* Object on left */
	  {
		ir_active = TRUE;
		ir_command = ARC_LEFT;	 /* Arc to the left */
		if (sound_inhibit == 0)
			tone(2000.,.05); 
	  }

	  ir_out= ir;
      /* sleep(0.1); */
	  defer();
    }
}



/* check bumpers */
int bump_out=0;

void bump()
{ 
	int bump_stat;

	while (1) {
		bump_stat=bumper();
		if (bump_stat == BUMP_BLR)	/* STOP */
		{ 
			bump_active = TRUE;
			bump_command = STOP;
		}
		if (bump_stat == BUMP_LR)
		{ 
			bump_active = TRUE;
			bump_command = BACKWARD;
			wait(MSEC_PER_RAD);		/* Move back a bit */
			bump_command = TURN_LEFT;
			wait(REV_4); 
		}
		else if (bump_stat == BUMP_LEFT)
		{ 
			bump_active = TRUE;
			bump_command = TURN_RIGHT;
			wait(REV_8); 			
		}
		else if (bump_stat == BUMP_RIGHT)
		{ 
			bump_active = TRUE;
			bump_command = TURN_LEFT;
			wait(REV_8); 
		}
		else if (bump_stat == BUMP_BACK)
		{ 
			bump_active = TRUE;
			bump_command = TURN_LEFT;
			wait(REV_4); 
			warble(1500,300,-60);
		}
		else
			bump_active = FALSE;

		bump_out=bump_stat;
		defer();
	}
}	

/* Print status info to the screen */
void printer()
{ 
	while (1)
	{
      		printf("\nIR: %d   ", ir_out);
      		printf("Bmp: %d   ", bump_out);
      		printf("Pho:%d ", photo_out);
      		printf("Snd:%d", sound_out);
      		wait(sleep_time*4); 
	}
}

/* CHECKS FOR A LOUD SOUND */
int sound_thresh = 200;
int sound_out;

void sound ()
{ 
	int val;

	while (1)
    { 
		val=analog(MICRO);
		if (val> sound_thresh)
		{
			warble(1500,300,-60); /* Acknowledge */
			sound_active=!sound_active; /* togle value */
			sound_command= STOP;
			sound_out=val;
			wait(1000);		/* stop for 1 seconds before hear again */
		}
		else 
			defer();
    }
}


void main()
{  
   printf("--UMH--Warrior--");
   printf("----En Accion---\n");
   sleep(2.0);
   test_number++;			/* Update test number */
   poke(IO_DIR, 0x0);		/* switch off lights */
   alert_tune();			/* Show that processes are starting */

   start_process(printer());		/* For debugging */
   start_process(motor_control());
   start_process(cruise());
   /*start_process(photo()); */
   start_process(ir_avoid());
   start_process(bump());
   start_process(sound());	/* Sometimes we listen for a sound */
 
}
