/*
 ********************************************************
	Rug Warrior Pro UMH-ISA (SITR)
 ********************************************************
	NOMBRE DEL ARCHIVO:   	motores.c
	FECHA:			
	AUTOR:   Luis M. Jimnez
	DESCRIPCIN:  Manejo de los motores por comandos.
					Driver diferencial con correccin de desviacin
					necesita: defines.c
				  
	VARIABLES GLOBALES EXTERNAS:
		int DRIVE_BIAS = 0;	
		int STOP       = 0;
		int FORWARD    = 1;
		int BACKWARD   = 2;
		int TURN_LEFT  = 3;
		int TURN_RIGHT = 4;
		int ARC_LEFT   = 5;
		int ARC_RIGHT  = 6;
		int SLOW_FORWARD    = 7;
		int SLOW_BACKWARD   = 8;
		int SLOW_TURN_LEFT  = 9;
		int SLOW_TURN_RIGHT = 10;

		int MSEC_PER_REV = 1071;
		int MSEC_PER_RAD = 170; 
		int REV_2 = MSEC_PER_REV / 2;
		int REV_4 = MSEC_PER_REV / 4;
		int REV_8 = MSEC_PER_REV / 8;

    FUNCIONES EXTERNAS: 
		void driveb(int trans, int rot);
		void move(int operation);
		int track(int trans_vel, int rot_vel, int clicks);

	VARIABLES GLOBALES USO INTERNO:
		int VEL_TRAS = 85;
		int VEL_TURN = 85;
		int VEL_SLOW_TRAS = 40;
		int VEL_SLOW_TURN = 40;
		int VEL_TRAS_ARC = 50;
		int VEL_TURN_ARC = 35;

		float STOP_TIME = 0.035;

    FUNCIONES USO INTERNO: 
 ********************************************************
*/

/* No two drive motors respond in exactly the same way to the same
   applied voltage.  Use the drive_bias term to correct for biases
   in your robot.  If your robot arcs to the right, make drive_bias
   positive, arcs to the left require a negative correction. */

int DRIVE_BIAS = 10;	/* Open loop correction term for drive motors */

int STOP       = 0;
int FORWARD    = 1;
int BACKWARD   = 2;
int TURN_LEFT  = 3;
int TURN_RIGHT = 4;
int ARC_LEFT   = 5;
int ARC_RIGHT  = 6;
int SLOW_FORWARD    = 7;
int SLOW_BACKWARD   = 8;
int SLOW_TURN_LEFT  = 9;
int SLOW_TURN_RIGHT = 10;


int VEL_TRAS = 85;
int VEL_TURN = 85;
int VEL_SLOW_TRAS = 40;
int VEL_SLOW_TURN = 40;
int VEL_TRAS_ARC = 50;
int VEL_TURN_ARC = 35;

/* Experimental data, 9 volts, concrete */
int MSEC_PER_REV = 1071; /* 714; /* mSeconds to make 1 revolution */
int MSEC_PER_RAD = 170;  /* 113; /* mSeconds to move 1 robot radius */
int REV_2 = MSEC_PER_REV / 2;   /* 180 degrees */
int REV_4 = MSEC_PER_REV / 4;	/* 90 degrees */ 
int REV_8 = MSEC_PER_REV / 8;	/* 45 degrees */

/* Corrige del desfase (bias) entre motores */
/* tambin chequea los limites ( no utilizar drive) */
/* trans + (rot + rot_bias)<100)  */
void driveb(int trans, int rot)	
{ 
	int rot_bias = (DRIVE_BIAS * trans) / 100;
	motor(MOTOR_LEFT,limit(trans - (rot + rot_bias), -100, 100));
	motor(MOTOR_RIGHT,limit(trans + (rot + rot_bias), -100, 100));
}

/* movimientos estndar */
void move(int operation)
{     
	if (operation == STOP) 				/* 0 - Stop */
		stop();
    else if (operation == FORWARD)		/* 1 - Forward */
		driveb(VEL_TRAS,0);
    else if (operation == BACKWARD)		/* 2 - Backward */
		driveb(-VEL_TRAS,0);
    else if (operation == TURN_LEFT)	/* 3 - Left Turn */
		driveb(0,VEL_TURN);
    else if (operation == TURN_RIGHT)	/* 4 - Right Turn */
		driveb(0,-VEL_TURN);

    else if (operation == ARC_LEFT)		/* 5 - Left Arc turn */
		driveb(VEL_TRAS_ARC,VEL_TURN_ARC);
    else if (operation == ARC_RIGHT)	/* 6 - Right Arc turn */
		driveb(VEL_TRAS_ARC,-VEL_TURN_ARC);

    else if (operation == SLOW_FORWARD)		/* 7 - Slow Forward */
		driveb(VEL_SLOW_TRAS,0);
    else if (operation == SLOW_BACKWARD)	/* 8 - Slow Backward */
		driveb(-VEL_SLOW_TRAS,0);
    else if (operation == SLOW_TURN_LEFT)	/* 9 - Slow Left Turn */
		driveb(0,VEL_SLOW_TURN);
    else if (operation == SLOW_TURN_RIGHT)	/* 10 - Slow Right Turn */
		driveb(0,-VEL_SLOW_TURN);
}


/* Track - Move a specified number of encoder clicks */
/* init_velocity must be called before calling track */

float STOP_TIME = 0.035;   /* Approximately, one servo cycle */

/* Track - drive at selected velocity until CLICKS encoder clicks have
   occurred, then stop.  
   Track works best with either trans_vel = 0, or rot_vel = 0.
 */
int track(int trans_vel, int rot_vel, int clicks)
{   
	long time_out = mseconds() + 5000L;     /* Time out after 5 seconds */
    int l_tot_clks = 0;    /* Total clicks */
    int r_tot_clks = 0;
    int ave_clks = 0;


    get_left_clicks();     /* Reset clicks */
    get_right_clicks();    /* Reset clicks */
    driveb(trans_vel, rot_vel);   /* Turn motors on */

    while(1)
    {
         l_tot_clks = l_tot_clks + get_left_clicks();  /* Count up the clicks */
         r_tot_clks = r_tot_clks + get_right_clicks();
         ave_clks = min(l_tot_clks, r_tot_clks);       /* Trigger on ave_clicks */
         if ((ave_clks >= clicks) || (mseconds() > time_out))
         {   
		 driveb((- trans_vel), (- rot_vel));   /* Try to kill motion */
             sleep(STOP_TIME);
		 stop();
             return (clicks - ave_clks);
         }
    }
}
